"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
/** @jsx jsx */
var core_1 = require("@emotion/core");
var React = tslib_1.__importStar(require("react"));
var use_positioner_1 = require("./Hooks/use-positioner");
var use_uid_1 = require("./Hooks/use-uid");
var compose_bind_1 = require("./Hooks/compose-bind");
var Providers_1 = require("./Theme/Providers");
var Text_1 = require("./Text");
var Touchable_1 = require("./Touchable");
var use_measure_1 = require("./Hooks/use-measure");
var Layer_1 = require("./Layer");
var react_highlight_words_1 = tslib_1.__importDefault(require("react-highlight-words"));
exports.ComboBoxContext = React.createContext(null);
exports.ComboBox = function (_a) {
    var children = _a.children, onSelect = _a.onSelect, query = _a.query, onQueryChange = _a.onQueryChange;
    var inputRef = React.useRef(null);
    var listRef = React.useRef(null);
    var listId = "list" + use_uid_1.useUid();
    var options = React.useRef([]);
    var position = use_positioner_1.usePositioner({
        modifiers: {
            flip: {
                enabled: false
            }
        }
    });
    var _b = React.useState(false), expanded = _b[0], setExpanded = _b[1];
    var _c = React.useState(null), selected = _c[0], setSelected = _c[1];
    var inputSize = use_measure_1.useMeasure(inputRef);
    /**
     * Handle input change
     */
    var onInputChange = React.useCallback(function (e) {
        // potentially show popover
        setSelected(null);
        if (!expanded) {
            setExpanded(true);
        }
        onQueryChange(e.target.value);
    }, [expanded]);
    /**
     * Escape key pressed
     */
    var onEscape = React.useCallback(function () {
        setExpanded(false);
        setSelected(null);
    }, []);
    /**
     * Enter pressed or item clicked
     */
    var onItemSelect = React.useCallback(function () {
        // call the parent with the selected value?
        setExpanded(false);
        onSelect && onSelect(selected);
        setSelected(null);
    }, [selected]);
    /**
     * Get the currently active option index
     * */
    var getSelectedIndex = React.useCallback(function () {
        if (!selected)
            return -1;
        return options.current.indexOf(selected || "");
    }, [options, selected]);
    /**
     * Arrow up pressed
     */
    var onArrowUp = React.useCallback(function () {
        var opts = options.current;
        var i = getSelectedIndex();
        // on input? cycle to bottom
        if (i === -1) {
            setSelected(opts[opts.length - 1]);
            // select prev
        }
        else {
            setSelected(opts[i - 1]);
        }
    }, [getSelectedIndex]);
    /**
     * Arrow down pressed
     */
    var onArrowDown = React.useCallback(function () {
        var opts = options.current;
        var i = getSelectedIndex();
        // if last, cycle to first
        if (i + 1 === opts.length) {
            setSelected(opts[0]);
            // or next
        }
        else {
            setSelected(opts[i + 1]);
        }
    }, [getSelectedIndex, selected]);
    /**
     * Handle keydown events
     */
    var onKeyDown = React.useCallback(function (e) {
        switch (e.key) {
            case "ArrowUp":
                e.preventDefault();
                onArrowUp();
                break;
            case "ArrowDown":
                e.preventDefault();
                onArrowDown();
                break;
            case "Escape":
                e.preventDefault();
                onEscape();
                break;
            case "Enter":
                e.preventDefault();
                onItemSelect();
                break;
        }
    }, [onArrowDown, onArrowUp, onEscape, onItemSelect]);
    /**
     * Handle blur events
     */
    var handleBlur = React.useCallback(function () {
        requestAnimationFrame(function () {
            var focusedElement = document.activeElement;
            var list = listRef.current;
            if (focusedElement == inputRef.current || focusedElement == list) {
                // ignore
                return;
            }
            // ignore if our popover contains the focused element
            if (list && list.contains(focusedElement)) {
                return;
            }
            // hide popover
            setExpanded(false);
            setSelected(null);
        });
    }, []);
    /**
     * handle input focus
     */
    var handleFocus = React.useCallback(function () {
        setExpanded(true);
    }, []);
    /**
     * Handle item clicks
     */
    var handleOptionSelect = React.useCallback(function (value) {
        onSelect && onSelect(value);
        setExpanded(false);
        setSelected(null);
    }, []);
    /**
     * Make a unique hash for list + option
     */
    var makeHash = React.useCallback(function (i) {
        return listId + i;
    }, [listId]);
    return (core_1.jsx(exports.ComboBoxContext.Provider, { value: {
            listId: listId,
            inputRef: inputRef,
            listRef: listRef,
            options: options,
            onInputChange: onInputChange,
            selected: selected,
            onKeyDown: onKeyDown,
            handleBlur: handleBlur,
            handleFocus: handleFocus,
            handleOptionSelect: handleOptionSelect,
            position: position,
            makeHash: makeHash,
            expanded: expanded,
            inputSize: inputSize.bounds,
            query: query
        } }, children));
};
exports.ComboBoxInput = function (_a) {
    var _b = _a.component, Component = _b === void 0 ? "input" : _b, _c = _a.autocomplete, autocomplete = _c === void 0 ? false : _c, other = tslib_1.__rest(_a, ["component", "autocomplete"]);
    var context = React.useContext(exports.ComboBoxContext);
    var _d = React.useState(""), localValue = _d[0], setLocalValue = _d[1];
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    var onKeyDown = context.onKeyDown, makeHash = context.makeHash, selected = context.selected, position = context.position, handleBlur = context.handleBlur, query = context.query, handleFocus = context.handleFocus, onInputChange = context.onInputChange, listId = context.listId, inputRef = context.inputRef;
    /** support autocomplete on selection */
    React.useEffect(function () {
        if (!autocomplete) {
            return;
        }
        if (selected) {
            setLocalValue(selected);
        }
        else {
            setLocalValue("");
        }
    }, [selected, autocomplete]);
    return (core_1.jsx(Component, tslib_1.__assign({ id: listId, onKeyDown: onKeyDown, onChange: onInputChange, onBlur: handleBlur, onFocus: handleFocus, "aria-controls": listId, autoComplete: "off", value: localValue || query, "aria-readonly": true, "aria-autocomplete": "list", role: "textbox", "aria-activedescendant": selected ? makeHash(selected) : undefined }, compose_bind_1.safeBind({
        ref: inputRef
    }, {
        ref: position.target.ref
    }, other))));
};
exports.ComboBoxList = function (_a) {
    var children = _a.children, _b = _a.autoHide, autoHide = _b === void 0 ? true : _b, other = tslib_1.__rest(_a, ["children", "autoHide"]);
    var context = React.useContext(exports.ComboBoxContext);
    var theme = Providers_1.useTheme();
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    var inputSize = context.inputSize, expanded = context.expanded, listId = context.listId, handleBlur = context.handleBlur, listRef = context.listRef, position = context.position, options = context.options;
    React.useLayoutEffect(function () {
        options.current = [];
        return function () {
            options.current = [];
        };
    });
    return (core_1.jsx(React.Fragment, null, (expanded || !autoHide) && (core_1.jsx(Layer_1.Layer, tslib_1.__assign({ tabIndex: -1, elevation: "sm", key: "1", style: position.popover.style, "data-placement": position.popover.placement, id: listId, role: "listbox", onBlur: handleBlur, className: "ComboBoxList", css: {
            overflow: "hidden",
            borderRadius: theme.radii.sm,
            outline: "none",
            width: inputSize.width +
                inputSize.left +
                (inputSize.right - inputSize.width) +
                "px",
            margin: 0,
            padding: 0
        } }, compose_bind_1.safeBind({
        ref: listRef
    }, {
        ref: position.popover.ref
    }, other)),
        children,
        core_1.jsx("div", { ref: position.arrow.ref, style: position.arrow.style })))));
};
exports.ComboBoxOption = function (_a) {
    var value = _a.value, children = _a.children, other = tslib_1.__rest(_a, ["value", "children"]);
    var context = React.useContext(exports.ComboBoxContext);
    var theme = Providers_1.useTheme();
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    var makeHash = context.makeHash, handleOptionSelect = context.handleOptionSelect, options = context.options, selected = context.selected;
    React.useEffect(function () {
        if (options.current) {
            options.current.push(value);
        }
    });
    var isSelected = selected === value;
    var onClick = React.useCallback(function () {
        handleOptionSelect(value);
    }, [value]);
    return (core_1.jsx(Touchable_1.Touchable, tslib_1.__assign({ tabIndex: -1, id: makeHash(value), role: "option", component: "li", onPress: onClick, "aria-selected": isSelected ? "true" : "false", css: {
            outline: "none",
            width: "100%",
            boxSizing: "border-box",
            display: "block",
            listStyleType: "none",
            margin: 0,
            padding: "0.5rem 0.75rem",
            cursor: "pointer",
            background: isSelected ? theme.colors.background.tint1 : "none",
            "&.Touchable--hover": {
                background: theme.colors.background.tint1
            },
            "&.Touchable--active": {
                background: theme.colors.background.tint2
            }
        } }, other), children || core_1.jsx(exports.ComboBoxOptionText, { value: value })));
};
exports.ComboBoxOptionText = function (_a) {
    var value = _a.value, other = tslib_1.__rest(_a, ["value"]);
    var context = React.useContext(exports.ComboBoxContext);
    var theme = Providers_1.useTheme();
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    var query = context.query;
    return (core_1.jsx(Text_1.Text, tslib_1.__assign({ className: "ComboBoxOptionText", css: { fontWeight: theme.fontWeights.heading } }, other),
        core_1.jsx(react_highlight_words_1.default, { highlightStyle: {
                color: theme.colors.text.selected,
                background: "transparent"
            }, className: "ComboBoxOptionText__Highlighter", searchWords: [query], autoEscape: true, textToHighlight: value })));
};
